"""UUID data support"""

import cgi

from xml.dom import minidom
from google.appengine.ext import db
from google.appengine.ext.db import polymodel
from datetime import datetime

uploads_map = {}

class UploadBatch(object):
  def __init__(self, upload_id, input_name, description,
               uploaded=False, enabled=False, wiped=False):
    self.upload_id = upload_id
    self.input_name = input_name
    self.description = description
    self.uploaded = uploaded
    self.enabled = enabled
    self.wiped = wiped

  def register(self):
    assert not uploads_map.has_key(self.upload_id)
    uploads_map[self.upload_id] = self

UploadBatch(1, "data/wine11.xml.gz", "Wine 1.1.0 include directory",
            uploaded=True, enabled=True).register()
UploadBatch(2, "data/wdk6.xml.gz", "Windows SDK 6.0",
            uploaded=True, enabled=True).register()
UploadBatch(3, "data/reg-wine11.xml.gz", "Wine 1.1.0 registry",
            uploaded=True, enabled=True).register()
UploadBatch(4, "data/reg-win98.xml.gz", "Windows 98 registry",
            uploaded=True, enabled=True).register()
UploadBatch(5, "data/reg-vista.xml.gz", "Windows Vista registry",
            uploaded=True, enabled=True).register()

class UuidEntry(polymodel.PolyModel):
  uuid = db.StringProperty(required=True)
  upload_id = db.IntegerProperty(required=True)
  source_name = db.StringProperty(required=True)
  
  def TypeSortKey(self):
    # Default: random order
    return None

class RegValue(UuidEntry):
  reg_key = db.StringProperty(required=True)
  
  def HTMLDescription(self):
    return ('Value of registry key <tt class="result">%s</tt>'
            % cgi.escape(self.reg_key))
  
  def TypeSortKey(self):
    return self.reg_key.lower()

class RegSubKey(UuidEntry):
  reg_key = db.StringProperty(required=True)
  
  def HTMLDescription(self):
    return ('Subkey of registry key <tt class="result">%s</tt>'
            % cgi.escape(self.reg_key))
  
  def TypeSortKey(self):
    return self.reg_key.lower()

class RegIID(UuidEntry):
  interface = db.StringProperty()
  
  def HTMLDescription(self):
    return ('Registered interface: <tt class="result">%s</tt>'
            % cgi.escape(self.interface))

class RegCLSID(UuidEntry):
  com_class_name = db.StringProperty()
  server_file = db.StringProperty()
  product = db.StringProperty()
  product_version = db.StringProperty()
  
  def HTMLDescription(self):
    result = ('Registered class: <tt class="result">%s</tt>'
              % cgi.escape(self.com_class_name))
    if self.server_file:
      result += ('<ul><li>Inproc sever: <tt class="result">%s</tt>' %
                 cgi.escape(self.server_file))
      if self.product:
        result += (
            (' (product: <span class="result">%s</span>,'
             'version <tt>%s</tt>)') %
             (cgi.escape(self.product), cgi.escape(str(self.product_version))))
      result += "</ul></li>\n"
    return result

class HeaderDefine(UuidEntry):
  filename = db.StringProperty(required=True)
  identifier = db.StringProperty()
  
  def HTMLDescription(self):
    return (('Declared as <tt class="result">%s</tt> in file '
             '<tt class="result">%s</tt>') %
	     (cgi.escape(self.identifier), cgi.escape(self.filename)))
  
  def TypeSortKey(self):
    return self.filename.lower()

class HeaderDeclspec(UuidEntry):
  filename = db.StringProperty(required=True)
  identifier = db.StringProperty()
  
  def HTMLDescription(self):
    if self.identifier:
      return (('Declared with <tt>__declspec(uuid)</tt> as '
               '<tt class="result">%s</tt> in file '
               '<tt class="result">%s</tt>') %
               (cgi.escape(self.identifier), cgi.escape(self.filename)))
    else:
      return (('Declared with <tt>__declspec(uuid)</tt> in file '
               '<tt class="result">%s</tt>') %
               (cgi.escape(self.filename)))
  
  def TypeSortKey(self):
    return self.filename.lower()


def ParseXMLFile(xml, upload_id):
  errors = []
  warnings = []
  results = []
  root = xml.documentElement
  source_name = root.getAttribute("name")
  for node in root.childNodes:
    if node.nodeType != minidom.Node.ELEMENT_NODE:
      continue
    name = node.localName
    entry = None
    if name == 'reg-value':
      entry = RegValue(uuid = node.getAttribute('uuid'),
                       upload_id = upload_id,
		       source_name = source_name,
                       reg_key = node.getAttribute('key'));
    elif name == 'reg-subkey':
      entry = RegSubKey(uuid = node.getAttribute('uuid'),
                        upload_id = upload_id,
		        source_name = source_name,
                        reg_key = node.getAttribute('key'));
    elif name == 'reg-clsid':
      entry = RegCLSID(uuid = node.getAttribute('uuid'),
                       upload_id = upload_id,
		       source_name = source_name,
                       com_class_name = node.getAttribute('name'));
      for child in node.childNodes:
        if child.nodeType == minidom.Node.ELEMENT_NODE:
          if child.localName == 'inproc-server':
            entry.server_file = child.getAttribute('file')
            if child.getAttribute('product'):
              entry.product = child.getAttribute('product')
            if child.getAttribute('product-version'):
              entry.product_version = child.getAttribute('product-version')
          else:
            errors += "Unknown CLSID child element " + child.localName
    elif name == 'reg-iid':
      entry = RegIID(uuid = node.getAttribute('uuid'),
                     upload_id = upload_id,
                     source_name = source_name,
                     interface = node.getAttribute('name'));
    elif name == 'definition':
      entry = HeaderDefine(uuid = node.getAttribute('uuid'),
                           upload_id = upload_id,
                           source_name = source_name,
                           filename = node.getAttribute('file'),
                           identifier = node.getAttribute('name'));
    elif name == 'declspec':
      entry = HeaderDeclspec(uuid = node.getAttribute('uuid'),
                             upload_id = upload_id,
                             source_name = source_name,
                             filename = node.getAttribute('file'),
                             identifier = node.getAttribute('name'));
    else:
      errors += 'Unknown element:' + name
    if entry != None:
      results += [entry]
#  batch_row = UploadBatch(upload_id = upload_id,
#                          source_name = source_name,
#			  entries_count = len(results))
#  results += [ batch_row ]
  return (results, errors, warnings)
